#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# Test documentation examples
#

import os, sys
import traceback
import unittest
import shutil
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

import albatross

this_dir = os.path.dirname(__file__)
example_dir = os.path.abspath(os.path.join(this_dir, os.pardir, 'doc', 'doctest'))

PS1 = '>>>'
PS2 = '...'


# Fake "fakeapp" module for examples to import
class FakeApp:
    class Request:
        def get_uri(self):
            return 'http://www.example.com/fakeapp.py'
        def write_header(self, name, value):
            pass
        def end_headers(self):
            pass
        def write_content(self, data):
            sys.stdout.write(data)
    app = albatross.SimpleApp(base_url='fakeapp.py',
                              template_path='.',
                              start_page='start',
                              secret='secret')
    ctx = albatross.SessionAppContext(app)
    ctx.set_request(Request())
sys.modules['fakeapp'] = FakeApp


class ParseError(Exception):
    pass


class ExampleCmd:
    """
    Statement (with continuation lines) and expected output
    """

    def __init__(self, linenum, line):
        self.linenum = linenum
        self.cmds = [line]
        self.expect = []

    def add_continuation(self, line):
        self.cmds.append(line)

    def has_output(self):
        return bool(self.expect)

    def add_output(self, line):
        self.expect.append(line)

    def run(self, namespace):
        got = None
        expect = ''.join(self.expect)
        src = ''.join(self.cmds)
        try:
            saved_stdout = sys.stdout
            sys.stdout = StringIO()
            try:
                code = compile(src, '<string>', 'single')
                exec code in namespace
                got = sys.stdout.getvalue()
            finally:
                sys.stdout = saved_stdout
        except:
            # Is it an expected exception?
            if ('Traceback (innermost last):\n' in expect or
                'Traceback (most recent call last):\n' in expect):
                # Only compare exception type and value - the rest of
                # the traceback isn't necessary.
                exc_type, exc_val = sys.exc_info()[:2]
                got = traceback.format_exception_only(exc_type, exc_val)[-1]
                expect = self.expect[-1]
            else:
                # Unexpected exception - print something useful
                exc_type, exc_val, exc_tb = sys.exc_info()
                exc_tb = exc_tb.tb_next
                psrc = PS1 + " " + src.rstrip().replace("\n", PS2 + " ")
                pexp = traceback.format_exception(exc_type, exc_val, exc_tb)
                pexp = ''.join(pexp).rstrip()
                raise AssertionError('Line %d:\n%s\n%s' % 
                                     (self.linenum, psrc, pexp))
        if got != expect:
            raise AssertionError('Line %d: output does not match example\n'
                                 'Expected:\n%s\nGot:\n%s' % 
                                 (self.linenum, expect, got))


class ExampleTest(unittest.TestCase):

    def __init__(self, name, filepath):
        unittest.TestCase.__init__(self)
        self.filepath = filepath
        cls = self.__class__
        self._testMethodDoc = '%s (%s.%s)' % (name, cls.__module__, cls.__name__)

    def load(self):
        self.example = []
        f = open(self.filepath)
        try:
            for linenum, line in enumerate(f):
                if line.startswith(PS1):
                    self.example.append(ExampleCmd(linenum + 1, 
                                                   line[len(PS1)+1:]))
                elif line.startswith(PS2):
                    if not self.example or self.example[-1].has_output():
                        raise ParseError("Line %d: %r line with no preceeding %r line" % (linenum+1, PS2, PS1))
                    self.example[-1].add_continuation(line[len(PS2)+1:])
                else:
                    if not self.example:
                        raise ParseError("Line %d: output with no preceeding %r line" % (linenum+1, PS1))
                    self.example[-1].add_output(line)
        finally:
            f.close()

    def setUp(self):
        # Some doc examples want to write files - make a scratch directory and
        # change into it.
        thisdir = os.path.dirname(__file__)
        self.scratchdir = os.path.join(thisdir, '.doc_examples_work')
        self.curdir = os.getcwd()
        os.mkdir(self.scratchdir)
        os.chdir(self.scratchdir)

    def tearDown(self):
        # Return to the original cwd and remove the scratch directory.
        os.chdir(self.curdir)
        assert self.scratchdir.endswith('.doc_examples_work')
        shutil.rmtree(self.scratchdir)

    def runTest(self):
        self.load()
        namespace = {}
        for cmd in self.example:
            cmd.run(namespace)


class suite(unittest.TestSuite):

    def __init__(self):
        unittest.TestSuite.__init__(self)
        files = os.listdir(example_dir)
        files.sort()
        for fn in files:
            fp = os.path.join(example_dir, fn)
            if not fn.startswith('.') and os.path.isfile(fp):
                self.addTest(ExampleTest(fn, fp))
                

if __name__ == '__main__':
    unittest.main(defaultTest='suite')
